package com.kexun.service.atomic;

import com.alibaba.fastjson.JSONObject;
import com.kexun.constant.CommonTypeConsts;
import com.kexun.entity.FieldDefineEntity;
import com.kexun.model.xml.*;
import com.kexun.model.xml.pub.*;
import com.kexun.model.xml.trs.*;
import com.kexun.utils.BaseUtils;
import com.kexun.utils.DateUtils;
import com.kexun.utils.ReflexUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Description: 基础字段-身份信息-原子算法service
 *
 * @author lgs
 * @date 2023/07/03
 */
@Service
public class BaseFieldAtomicService {

    /**
     *         "性别": "男",
     *         "出生日期": "1989-06-28",
     *         "国籍": "中国",
     *         "户籍地址": "",
     *         "婚姻状况": "已婚",
     *         "学历": "本科",
     *         "学位": "",
     *         "通讯地址": "河北省衡水市安平县",
     *         "就业状况": "在职",
     *         "历史手机号码数": 2,
     *         "近3个月手机号码数": 0,
     *         "近6个月手机号码数": 1,
     *         "近12个月手机号码数": 2,
     *         "近24个月手机号码数": 2
     */

    /**
     * 对数据库中配置的各个字段的值进行赋值，并保存到相应的Map集合中
     *
     * @param fieldDefineEntity 数据库字段定义类
     * @param jsonResultMap     Map集合
     * @param list              数据源List
     */
    private void setJsonResultMap(FieldDefineEntity fieldDefineEntity, Map<String, String> jsonResultMap, List<? extends Object> list) {
        String path = fieldDefineEntity.getPath();
        String name = fieldDefineEntity.getName();
        String type = fieldDefineEntity.getType();
        String value = "";
        if (null != list && list.size() > 0) {
            value = ReflexUtils.getAnnotaionFiledValue(list.get(0), path);
        }
        jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
    }

    /**
     * 公共信息明细-查询记录
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public List<Object> queryRecordCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        List jsonResultList = new ArrayList();
        QueryRecord queryRecord = jsonObject.toJavaObject(QueryRecord.class);
        if (null != queryRecord) {
            List<QueryRecordInf> queryRecordInfList = queryRecord.getQueryRecordInfList();

            if (null != queryRecordInfList && queryRecordInfList.size() > 0) {
                List<Map> trsInfoList = new ArrayList<>();
                for (QueryRecordInf queryRecordInf : queryRecordInfList) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefineEntity : defineEntityList) {
                        String path = fieldDefineEntity.getPath();
                        String name = fieldDefineEntity.getName();
                        String type = fieldDefineEntity.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(queryRecordInf, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultList.addAll(trsInfoList);
            }
        }
        return jsonResultList;
    }

    /**
     * 公共信息明细-行政奖励记录
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public List<Object> pubRewardCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        List jsonResultList = new ArrayList();
        PubReward pubReward = jsonObject.toJavaObject(PubReward.class);
        if (null != pubReward) {
            List<PubRewardInf> pubRewardInfList = pubReward.getPubRewardInfList();

            if (null != pubRewardInfList && pubRewardInfList.size() > 0) {
                List<Map> trsInfoList = new ArrayList<>();
                for (PubRewardInf pubRewardInf : pubRewardInfList) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefineEntity : defineEntityList) {
                        String path = fieldDefineEntity.getPath();
                        String name = fieldDefineEntity.getName();
                        String type = fieldDefineEntity.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(pubRewardInf, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultList.addAll(trsInfoList);
            }
        }
        return jsonResultList;
    }

    /**
     * 公共信息明细-执业资格记录
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public List<Object> pubPractisingCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        List jsonResultList = new ArrayList();
        PubPractising pubPractising = jsonObject.toJavaObject(PubPractising.class);
        if (null != pubPractising) {
            List<PubPractisingInf> pubPractisingInfList = pubPractising.getPubPractisingInfList();

            if (null != pubPractisingInfList && pubPractisingInfList.size() > 0) {
                List<Map> trsInfoList = new ArrayList<>();
                for (PubPractisingInf pubPractisingInf : pubPractisingInfList) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefineEntity : defineEntityList) {
                        String path = fieldDefineEntity.getPath();
                        String name = fieldDefineEntity.getName();
                        String type = fieldDefineEntity.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(pubPractisingInf, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultList.addAll(trsInfoList);
            }
        }
        return jsonResultList;
    }

    /**
     * 公共信息明细-行政处罚记录
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> pubAllowAncCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();
        PubAllowAnc pubAllowAnc = jsonObject.toJavaObject(PubAllowAnc.class);
        if (null != pubAllowAnc) {
            Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
            //需要计算的笔数
            List<FieldDefineEntity> fieldDefineCount = fieldDefineMap.get(CommonTypeConsts.COMMON_SUM);

            String isExistPubAllowAncKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_IS_EXIST))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            List<PubAllowAncInf> pubAllowAncInfList = pubAllowAnc.getPubAllowAncInfList();

            if (null != pubAllowAncInfList && pubAllowAncInfList.size() > 0) {
                jsonResultMap.put(isExistPubAllowAncKey, CommonTypeConsts.COMMON_IS_EXIST_YES);
                List<FieldDefineEntity> fieldDefineList = fieldDefineMap.get(CommonTypeConsts.PUB_ALLOW_ANC_LIST);
                List<Map> trsInfoList = new ArrayList<>();
                for (PubAllowAncInf pubAllowAncInf : pubAllowAncInfList) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefineEntity : fieldDefineList) {
                        String path = fieldDefineEntity.getPath();
                        String name = fieldDefineEntity.getName();
                        String type = fieldDefineEntity.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(pubAllowAncInf, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultMap.put(CommonTypeConsts.PUB_ALLOW_ANC_LIST, trsInfoList);

            } else {
                jsonResultMap.put(isExistPubAllowAncKey, CommonTypeConsts.COMMON_IS_EXIST_NO);
            }
        }
        return jsonResultMap;
    }

    /**
     * 公共信息明细-住房公积金参缴记录
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> pubHousingFundCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();
        PubHousingFund pubHousingFund = jsonObject.toJavaObject(PubHousingFund.class);
        if (null != pubHousingFund) {
            List<PubHousingFundInf> pubHousingFundInfList = pubHousingFund.getPubHousingFundInfList();
            if (pubHousingFundInfList != null && !pubHousingFundInfList.isEmpty()) {

                Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));

                //需要计算的笔数
                List<FieldDefineEntity> fieldDefineCount = fieldDefineMap.get(CommonTypeConsts.COMMON_SUM);

                List<FieldDefineEntity> fieldDefineList = fieldDefineMap.get(CommonTypeConsts.PUB_HOUSING_FUND_LIST);

                //住房公积金-最近1年公积金平均值
                String last1YearHousingFundAvgKey = fieldDefineCount.stream()
                        .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_1YEAR))
                        .map(FieldDefineEntity::getName)
                        .findFirst()
                        .orElse("");
                Date currtDate = new Date();
                List<PubHousingFundInf> last1YearHousingList = pubHousingFundInfList.stream()
                        .filter(e -> e.getEndMonth() != null &&
                                DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getEndMonth(), DateUtils.YYYY_MM), currtDate, DateUtils.UNIT_MOUTHS) <= 12)
                        .collect(Collectors.toList());
                BigDecimal last1YearHousingFund = last1YearHousingList.stream()
                        .map(e -> e.getMonthMoney() == null ? BigDecimal.ZERO : e.getMonthMoney())
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                BigDecimal last1YearHousingFundAvg = new BigDecimal("0.00");
                if (last1YearHousingFund.compareTo(BigDecimal.ZERO) > 0) {
                    last1YearHousingFundAvg = last1YearHousingFund.divide(BigDecimal.valueOf(last1YearHousingList.size()), 2, BigDecimal.ROUND_HALF_UP);
                }
                jsonResultMap.put(last1YearHousingFundAvgKey, last1YearHousingFundAvg);
                //住房公积金-最近3年公积金平均值
                String last3YearHousingFundAvgKey = fieldDefineCount.stream()
                        .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_3YEAR))
                        .map(FieldDefineEntity::getName)
                        .findFirst()
                        .orElse("");
                List<PubHousingFundInf> last3YearHousingList = pubHousingFundInfList.stream()
                        .filter(e -> e.getEndMonth() != null &&
                                DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getEndMonth(), DateUtils.YYYY_MM), currtDate, DateUtils.UNIT_MOUTHS) <= 36)
                        .collect(Collectors.toList());
                BigDecimal last3YearHousingFund = last3YearHousingList.stream()
                        .map(e -> e.getMonthMoney() == null ? BigDecimal.ZERO : e.getMonthMoney())
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                BigDecimal last3YearHousingFundAvg = new BigDecimal("0.00");

                if (last3YearHousingFund.compareTo(BigDecimal.ZERO) > 0) {
                    last3YearHousingFundAvg = last3YearHousingFund.divide(BigDecimal.valueOf(last3YearHousingList.size()), 2, BigDecimal.ROUND_HALF_UP);
                }
                jsonResultMap.put(last3YearHousingFundAvgKey, last3YearHousingFundAvg);

                List<Map> trsInfoList = new ArrayList<>();
                for (PubHousingFundInf pubHousingFundInf : pubHousingFundInfList) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefineEntity : fieldDefineList) {
                        String path = fieldDefineEntity.getPath();
                        String name = fieldDefineEntity.getName();
                        String type = fieldDefineEntity.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(pubHousingFundInf, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultMap.put(CommonTypeConsts.PUB_HOUSING_FUND_LIST, trsInfoList);

            }
        }
        return jsonResultMap;

    }

    /**
     * 公共信息明细-行政处罚记录
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> pubPenaltylCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();
        PubPenaltyl pubPenaltyl = jsonObject.toJavaObject(PubPenaltyl.class);
        if (null != pubPenaltyl) {
            List<PubPenaltylInf> pubPenaltylInfList = pubPenaltyl.getPubPenaltylInfList();
            Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));

            //需要计算的笔数
            List<FieldDefineEntity> fieldDefineCount = fieldDefineMap.get(CommonTypeConsts.COMMON_SUM);

            List<FieldDefineEntity> fieldDefineList = fieldDefineMap.get(CommonTypeConsts.PUB_PENALTYL_LIST);

            Date currtDate = new Date();
            //1、最近3个月行政处罚笔数
            String pubPenalty3MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_3MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubPenalty3MonCount = pubPenaltylInfList.stream()
                    .filter(e -> e.getPunStartDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getPunStartDate(), DateUtils.YYYY_MM), currtDate, DateUtils.UNIT_MOUTHS) <= 3)
                    .count();
            if (!StringUtils.isEmpty(pubPenalty3MonCountKey)) {
                jsonResultMap.put(pubPenalty3MonCountKey, pubPenalty3MonCount);
            }


            //2、最近6个月行政处罚笔数
            String pubPenalty6MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_6MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubPenalty6MonCount = pubPenaltylInfList.stream()
                    .filter(e -> e.getPunStartDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getPunStartDate(), DateUtils.YYYY_MM), currtDate, DateUtils.UNIT_MOUTHS) <= 6)
                    .count();
            if (!StringUtils.isEmpty(pubPenalty6MonCountKey)) {
                jsonResultMap.put(pubPenalty6MonCountKey, pubPenalty6MonCount);
            }

            //3、最近9个月行政处罚笔数
            String pubPenalty9MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_9MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubPenalty9MonCount = pubPenaltylInfList.stream()
                    .filter(e -> e.getPunStartDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getPunStartDate(), DateUtils.YYYY_MM), currtDate, DateUtils.UNIT_MOUTHS) <= 9)
                    .count();
            if (!StringUtils.isEmpty(pubPenalty9MonCountKey)) {
                jsonResultMap.put(pubPenalty9MonCountKey, pubPenalty9MonCount);
            }

            //4、最近12个月行政处罚笔数
            String pubPenalty12MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_12MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubPenalty12MonCount = pubPenaltylInfList.stream()
                    .filter(e -> e.getPunStartDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getPunStartDate(), DateUtils.YYYY_MM), currtDate, DateUtils.UNIT_MOUTHS) <= 12)
                    .count();
            if (!StringUtils.isEmpty(pubPenalty12MonCountKey)) {
                jsonResultMap.put(pubPenalty12MonCountKey, pubPenalty12MonCount);
            }

            //5、最近24个月行政处罚笔数
            String pubPenalty24MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_24MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubPenalty24MonCount = pubPenaltylInfList.stream()
                    .filter(e -> e.getPunStartDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getPunStartDate(), DateUtils.YYYY_MM), currtDate, DateUtils.UNIT_MOUTHS) <= 24)
                    .count();
            if (!StringUtils.isEmpty(pubPenalty24MonCountKey)) {
                jsonResultMap.put(pubPenalty24MonCountKey, pubPenalty24MonCount);
            }

            List<Map> trsInfoList = new ArrayList<>();
            for (PubPenaltylInf pubPenaltylInf : pubPenaltylInfList) {
                Map<String, String> baseInfoMap = new HashMap<>();
                for (FieldDefineEntity fieldDefineEntity : fieldDefineList) {
                    String path = fieldDefineEntity.getPath();
                    String name = fieldDefineEntity.getName();
                    String type = fieldDefineEntity.getType();
                    String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(pubPenaltylInf, path), type);
                    baseInfoMap.put(name, value);
                }
                trsInfoList.add(baseInfoMap);
            }
            jsonResultMap.put(CommonTypeConsts.PUB_PENALTYL_LIST, trsInfoList);

        }
        return jsonResultMap;
    }

    /**
     * 公共信息明细-强制执行记录
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> pubEnforcemenCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();
        PubEnforcemen pubEnforcemen = jsonObject.toJavaObject(PubEnforcemen.class);
        if (pubEnforcemen != null) {
            Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
            List<PubEnforcemenInf> pubEnforcemenInfList = pubEnforcemen.getPubEnforcemenInfList();
            //需要计算的笔数
            List<FieldDefineEntity> fieldDefineCount = fieldDefineMap.get(CommonTypeConsts.COMMON_SUM);
            List<FieldDefineEntity> fieldDefineList = fieldDefineMap.get(CommonTypeConsts.PUB_ENFORCEMEN_LIST);
            //1、总申请执行标的价值-key
            String pubEnforceExeMoneySumKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_COUNT))
                    .findFirst()
                    .map(FieldDefineEntity::getName)
                    .orElse("");
            //总申请执行标的价值-value
            BigDecimal pubEnforceExeMoneySum = pubEnforcemenInfList.stream()
                    .filter(e -> e.getAskExeMoney() != null)
                    .map(PubEnforcemenInf::getAskExeMoney)
                    .reduce(BigDecimal::add)
                    .get();
            if (!StringUtils.isEmpty(pubEnforceExeMoneySumKey)) {
                jsonResultMap.put(pubEnforceExeMoneySumKey, pubEnforceExeMoneySum);
            }
            //2、总已执行标的金额
            String pubEnforceExedObjAmtSumKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_SUM_AMT))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            //总已执行标的金额-value
            BigDecimal pubEnforceExedObjAmtSum = pubEnforcemenInfList.stream()
                    .filter(e -> e.getAskExeMoney() != null)
                    .map(PubEnforcemenInf::getExedObjAmt)
                    .reduce(BigDecimal::add)
                    .get();
            if (!StringUtils.isEmpty(pubEnforceExedObjAmtSumKey)) {
                jsonResultMap.put(pubEnforceExedObjAmtSumKey, pubEnforceExedObjAmtSum);
            }

            //3、历史是否存在未结案强制执行案件 -key --todo
            Date currtDate = new Date();
            //4、强制执行记录-最近3个月强制执行笔数
            String pubEnfLast3MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_3MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubEnfLast3MonCount = pubEnforcemenInfList.stream()
                    .filter(e -> e.getFilingDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getFilingDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 3)
                    .count();
            if (!StringUtils.isEmpty(pubEnfLast3MonCountKey)) {
                jsonResultMap.put(pubEnfLast3MonCountKey, pubEnfLast3MonCount);
            }

            //5、强制执行记录-最近6个月强制执行笔数
            String pubEnfLast6MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_6MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubEnfLast6MonCount = pubEnforcemenInfList.stream()
                    .filter(e -> e.getFilingDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getFilingDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 6)
                    .count();
            if (!StringUtils.isEmpty(pubEnfLast6MonCountKey)) {
                jsonResultMap.put(pubEnfLast6MonCountKey, pubEnfLast6MonCount);
            }
            //6、强制执行记录-最近9个月强制执行笔数
            String pubEnfLast9MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_9MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubEnfLast9MonCount = pubEnforcemenInfList.stream()
                    .filter(e -> e.getFilingDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getFilingDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 9)
                    .count();
            if (!StringUtils.isEmpty(pubEnfLast9MonCountKey)) {
                jsonResultMap.put(pubEnfLast9MonCountKey, pubEnfLast9MonCount);
            }
            //7、强制执行记录-最近12个月强制执行笔数
            String pubEnfLast12MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_12MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubEnfLast12MonCount = pubEnforcemenInfList.stream()
                    .filter(e -> e.getFilingDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getFilingDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 12)
                    .count();
            if (StringUtils.isEmpty(pubEnfLast12MonCountKey)) {
                jsonResultMap.put(pubEnfLast12MonCountKey, pubEnfLast12MonCount);
            }
            //8、强制执行记录-最近24个月强制执行笔数
            String pubEnfLast24MonCountKey = fieldDefineCount.stream()
                    .filter(e -> e.getPath().equals(CommonTypeConsts.COMMON_24MONTH))
                    .map(FieldDefineEntity::getName)
                    .findFirst()
                    .orElse("");
            long pubEnfLast24MonCount = pubEnforcemenInfList.stream()
                    .filter(e -> e.getFilingDate() != null &&
                            DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getFilingDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 24)
                    .count();
            if (StringUtils.isEmpty(pubEnfLast24MonCountKey)) {
                jsonResultMap.put(pubEnfLast24MonCountKey, pubEnfLast24MonCount);
            }

            List<Map> trsInfoList = new ArrayList<>();
            for (PubEnforcemenInf pubEnforcemenInf : pubEnforcemenInfList) {
                Map<String, String> baseInfoMap = new HashMap<>();
                for (FieldDefineEntity fieldDefineEntity : fieldDefineList) {
                    String path = fieldDefineEntity.getPath();
                    String name = fieldDefineEntity.getName();
                    String type = fieldDefineEntity.getType();
                    String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(pubEnforcemenInf, path), type);
                    baseInfoMap.put(name, value);
                }
                trsInfoList.add(baseInfoMap);
            }
            jsonResultMap.put(CommonTypeConsts.PUB_ENFORCEMEN_LIST, trsInfoList);

        }

        return jsonResultMap;
    }

    /**
     * 公共信息明细-民事判决记录
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> pubCivilCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();
        PubCivil pubCivil = jsonObject.toJavaObject(PubCivil.class);
        if (null != pubCivil) {
            List<PubCivilInf> pubCivilInfList = pubCivil.getPubCivilInfList();
            if (null != pubCivilInfList && pubCivilInfList.size() > 0) {

                //字段定义配置数据，对字段配置进行分组
                Map<String, List<FieldDefineEntity>> fieldDefineListMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
                List<FieldDefineEntity> fieldDefineSum = fieldDefineListMap.get(CommonTypeConsts.COMMON_SUM);
                if (fieldDefineSum != null && fieldDefineSum.size() > 0) {
                    Date currtDate = new Date();
                    //最近24个月民事判决笔数
                    long pubCivilLast24MonCount = pubCivilInfList.stream()
                            .filter(e -> e.getJudDate() != null &&
                                    DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getJudDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 24
                            ).count();
                    String name = fieldDefineSum.get(0).getName();
                    jsonResultMap.put(name, pubCivilLast24MonCount);
                }
                List<FieldDefineEntity> fieldDefinePubCiviList = fieldDefineListMap.get(CommonTypeConsts.PUB_CIVIL_LIST);
                List<Map> trsInfoList = new ArrayList<>();
                for (PubCivilInf pubCivilInf : pubCivilInfList) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefineEntity : fieldDefinePubCiviList) {
                        String path = fieldDefineEntity.getPath();
                        String name = fieldDefineEntity.getName();
                        String type = fieldDefineEntity.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(pubCivilInf, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultMap.put(CommonTypeConsts.PUB_CIVIL_LIST, trsInfoList);
            }
        }
        return jsonResultMap;
    }

    /**
     * 公共信息明细-欠税记录
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> pubOwingTaxesCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();
        PubOwingTaxes pubOwingTaxes = jsonObject.toJavaObject(PubOwingTaxes.class);
        if (null != pubOwingTaxes) {
            List<PubOwingTaxesInf> pubOwingTaxesInfList = pubOwingTaxes.getPubOwingTaxesInfList();
            if (pubOwingTaxesInfList != null && pubOwingTaxesInfList.size() > 0) {
                Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));

                //欠税记录-其它字段配置信息
                List<FieldDefineEntity> fieldDefineTaxesList = fieldDefineMap.get(CommonTypeConsts.PUB_OWING_TAXES_LIST);
                //循环欠税记录信息
                Date currtDate = new Date();
                //欠税记录-最近3个月欠税笔数
                long last3MonthTaxCount = pubOwingTaxesInfList.stream()
                        .filter(e -> e.getOwingTaxesDate() != null &&
                                DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getOwingTaxesDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 3
                        ).count();
                jsonResultMap.put(CommonTypeConsts.LAST_3MONTH_TAX_COUNT, last3MonthTaxCount);
                //欠税记录-最近6个月欠税笔数
                long last6MonthTaxCount = pubOwingTaxesInfList.stream()
                        .filter(e -> e.getOwingTaxesDate() != null &&
                                DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getOwingTaxesDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 6
                        ).count();
                jsonResultMap.put(CommonTypeConsts.LAST_6MONTH_TAX_COUNT, last6MonthTaxCount);
                //欠税记录-最近9个月欠税笔数
                long last9MonthTaxCount = pubOwingTaxesInfList.stream()
                        .filter(e -> e.getOwingTaxesDate() != null &&
                                DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getOwingTaxesDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 9
                        ).count();
                jsonResultMap.put(CommonTypeConsts.LAST_9MONTH_TAX_COUNT, last9MonthTaxCount);
                //欠税记录-最近12个月欠税笔数
                long last12MonthTaxCount = pubOwingTaxesInfList.stream()
                        .filter(e -> e.getOwingTaxesDate() != null &&
                                DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getOwingTaxesDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 12
                        ).count();
                jsonResultMap.put(CommonTypeConsts.LAST_12MONTH_TAX_COUNT, last12MonthTaxCount);
                //欠税记录-最近24个月欠税笔数
                long last24MonthTaxCount = pubOwingTaxesInfList.stream()
                        .filter(e -> e.getOwingTaxesDate() != null &&
                                DateUtils.getSpaceByUnit(DateUtils.convertStrToDate(e.getOwingTaxesDate(), DateUtils.YYYY_MM_DD), currtDate, DateUtils.UNIT_MOUTHS) <= 24
                        ).count();
                jsonResultMap.put(CommonTypeConsts.LAST_24MONTH_TAX_COUNT, last24MonthTaxCount);

                List<Map> trsInfoList = new ArrayList<>();
                for (PubOwingTaxesInf pubOwingTaxesInf : pubOwingTaxesInfList) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefineEntity : fieldDefineTaxesList) {
                        String path = fieldDefineEntity.getPath();
                        String name = fieldDefineEntity.getName();
                        String type = fieldDefineEntity.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(pubOwingTaxesInf, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultMap.put(CommonTypeConsts.PUB_OWING_TAXES_LIST, trsInfoList);
            }
        }
        return jsonResultMap;
    }

    /**
     * 非信贷交易明细信息
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public List<Object> nonCrditiTrsCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        List jsonResultList = new ArrayList();
        NctTrsInfo nctTrsInfo = jsonObject.toJavaObject(NctTrsInfo.class);
        if (null != nctTrsInfo) {
            List<NctTrsBase> nctTrsBaseList = nctTrsInfo.getNctTrsBaseList();
            if (null != nctTrsBaseList && nctTrsBaseList.size() > 0) {
                List<Map> trsInfoList = new ArrayList<>();
                for (NctTrsBase nctTrsBase : nctTrsBaseList) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefinePer : defineEntityList) {
                        String path = fieldDefinePer.getPath();
                        String name = fieldDefinePer.getName();
                        String type = fieldDefinePer.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(nctTrsBase, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultList.addAll(trsInfoList);
            }
        }
        return jsonResultList;
    }

    /**
     * 授信协议-交易明细
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> creditTrsInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();
        CtrctInfo ctrctInfo = jsonObject.toJavaObject(CtrctInfo.class);
        if (null != ctrctInfo) {
            List<CtrctBase> ctrctBaseList = ctrctInfo.getCtrctBaseList();
            if (null != ctrctBaseList && ctrctBaseList.size() > 0) {
                //协议账号统计
                int ctrctCount = ctrctBaseList.size();
                //字段定义配置数据，对字段配置进行分组
                Map<String, List<FieldDefineEntity>> fieldDefineListMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
                //计算账户的和，只有一条配置
                List<FieldDefineEntity> fieldDefineSum = fieldDefineListMap.get(CommonTypeConsts.COMMON_SUM);
                if (null != fieldDefineSum) {
                    String name = fieldDefineSum.get(0).getName();
                    jsonResultMap.put(name, ctrctCount);
                }
                List<Map> trsInfoList = new ArrayList<>();
                for (CtrctBase ctrctBase : ctrctBaseList) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefinePer : defineEntityList) {
                        String path = fieldDefinePer.getPath();
                        String name = fieldDefinePer.getName();
                        String type = fieldDefinePer.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(ctrctBase, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultMap.put(CommonTypeConsts.TRSINFO_CTRCT_LIST, trsInfoList);
            }
        }
        return jsonResultMap;
    }

    /**
     * 交易明细 相关还款责任人交易算法
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> relRepayInfoTrsInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();
        RltRepayInfo rltRepayInfo = jsonObject.toJavaObject(RltRepayInfo.class);
        if (null != rltRepayInfo) {
            List<RltRepay> rltRepayList = rltRepayInfo.getRltRepayList();
            //按照借款人为1-自然人和2-组织机构进行分类
            Map<String, List<RltRepay>> rltRepayMap = rltRepayList.stream().collect(Collectors.groupingBy(e -> e.getMainIdType()));
            //字段列表数据源配置信息
            Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
            List<FieldDefineEntity> fieldDefinePers = fieldDefineMap.get(CommonTypeConsts.ID_INFO_TYPE_PER);

            List<FieldDefineEntity> fieldDefineEnts = fieldDefineMap.get(CommonTypeConsts.ID_INFO_TYPE_ENT);
            //自然人列表
            List<RltRepay> rltRepayPers = rltRepayMap.get(CommonTypeConsts.ID_INFO_TYPE_PER);
            if (null != rltRepayPers && rltRepayPers.size() > 0) {
                List<Map> trsInfoList = new ArrayList<>();
                for (RltRepay rltRepayPer : rltRepayPers) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefinePer : fieldDefinePers) {
                        String path = fieldDefinePer.getPath();
                        String name = fieldDefinePer.getName();
                        String type = fieldDefinePer.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(rltRepayPer, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultMap.put(CommonTypeConsts.TRSINFO_REL_REPAY_PER, trsInfoList);

            }
            //组织机构列表
            List<RltRepay> rltRepayEnts = rltRepayMap.get(CommonTypeConsts.ID_INFO_TYPE_ENT);
            if (null != rltRepayEnts && rltRepayEnts.size() > 0) {
                List<Map> trsInfoList = new ArrayList<>();
                for (RltRepay rltRepayEnt : rltRepayEnts) {
                    Map<String, String> baseInfoMap = new HashMap<>();
                    for (FieldDefineEntity fieldDefineEnt : fieldDefineEnts) {
                        String path = fieldDefineEnt.getPath();
                        String name = fieldDefineEnt.getName();
                        String type = fieldDefineEnt.getType();
                        String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(rltRepayEnt, path), type);
                        baseInfoMap.put(name, value);
                    }
                    trsInfoList.add(baseInfoMap);
                }
                jsonResultMap.put(CommonTypeConsts.TRSINFO_REL_REPAY_ENT, trsInfoList);
            }
        }
        return jsonResultMap;
    }

    /**
     * 交易明细算法
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> trsInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();

        ForcedAcctTrsInf forcedAcctTrsInf = jsonObject.toJavaObject(ForcedAcctTrsInf.class);
        //数据源
        List<TrsAcctInf> trsAcctInfList = forcedAcctTrsInf.getTrsAcctInfList();
        //统计账号的个数
        int acctCount = trsAcctInfList.size();

        //字段定义配置数据，对字段配置进行分组
        Map<String, List<FieldDefineEntity>> fieldDefineListMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
        //计算账户的和，只有一条配置
        List<FieldDefineEntity> fieldDefineSum = fieldDefineListMap.get(CommonTypeConsts.COMMON_SUM);
        if (null != fieldDefineSum) {
            String name = fieldDefineSum.get(0).getName();
            jsonResultMap.put(name, acctCount);
        }
        //1、信贷交易明细信息-基本信息段
        List<FieldDefineEntity> defineEntitiesBase = fieldDefineListMap.get(CommonTypeConsts.TRSINFO_BASE);
        //2、信贷交易明细信息-最新表现信息
        List<FieldDefineEntity> fieldDefineTrsNew = fieldDefineListMap.get(CommonTypeConsts.TRSINFO_TRSNEW);
        //3、信贷交易明细信息-最近一次月度表现信息
        List<FieldDefineEntity> fieldDefineLateMonth = fieldDefineListMap.get(CommonTypeConsts.TRSINFO_LASTMONTH);
        //4、信贷交易明细信息-最近5年内历史表现信息
        List<FieldDefineEntity> fieldDefineFiveYear = fieldDefineListMap.get(CommonTypeConsts.TRSINFO_LASTFIVEYEAR);
        //5、特殊交易信息
        List<FieldDefineEntity> fieldDefineSpectrs = fieldDefineListMap.get(CommonTypeConsts.TRSINFO_SPECTRS);
        //6、特殊事件说明信息
        List<FieldDefineEntity> fieldDefineSpecEvt = fieldDefineListMap.get(CommonTypeConsts.TRSINFO_SPECEVT);
        //7、大额专项分期信息段
        List<FieldDefineEntity> fieldDefineSpecPrd = fieldDefineListMap.get(CommonTypeConsts.TRSINFO_SPECTRSPRD);
        //定义各分类账户存放的List
        List<Map> trsInfoList = new ArrayList<>();
        for (TrsAcctInf trsAcctInf : trsAcctInfList) {
            //以数据源为主体，循环数据源，有数据则进行组装JSON串
            Map<String, Map> trsInfMap = new HashMap<>();
            //基础信息段
            TrsBase trsBase = trsAcctInf.getTrsBase();
            Map<String, String> baseInfoMap = new HashMap<>();
            if (null != trsBase && null != defineEntitiesBase) {
                for (FieldDefineEntity fieldDefineEntity : defineEntitiesBase) {
                    String path = fieldDefineEntity.getPath();
                    String name = fieldDefineEntity.getName();
                    String type = fieldDefineEntity.getType();
                    String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(trsBase, path), type);
                    baseInfoMap.put(name, value);
                }
                trsInfMap.put(CommonTypeConsts.TRSINFO_BASE, baseInfoMap);
            }
            //最新表现信息
            TrsLate trsLate = trsAcctInf.getTrsLate();
            Map<String, String> trsNewMap = new HashMap<>();
            if (null != trsLate && null != fieldDefineTrsNew) {
                for (FieldDefineEntity fieldDefineEntity : fieldDefineTrsNew) {
                    String path = fieldDefineEntity.getPath();
                    String name = fieldDefineEntity.getName();
                    String type = fieldDefineEntity.getType();
                    String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(trsLate, path), type);
                    trsNewMap.put(name, value);
                }
                trsInfMap.put(CommonTypeConsts.TRSINFO_TRSNEW, trsNewMap);
            }
            //最近一次月度表现信息
            TrsLateMonth trsLateMonth = trsAcctInf.getTrsLateMonth();
            Map<String, String> trsLateMonthMap = new HashMap<>();
            if (null != trsLateMonth && null != fieldDefineLateMonth) {
                for (FieldDefineEntity fieldDefineEntity : fieldDefineLateMonth) {
                    String path = fieldDefineEntity.getPath();
                    String name = fieldDefineEntity.getName();
                    String type = fieldDefineEntity.getType();
                    String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(trsLateMonth, path), type);
                    trsLateMonthMap.put(name, value);
                }
                trsInfMap.put(CommonTypeConsts.TRSINFO_LASTMONTH, trsLateMonthMap);
            }
            //最近5年内历史表现信息
            TrsFiveYear trsFiveYear = trsAcctInf.getTrsFiveYear();
            Map<Object, Object> trsFiveYearMap = new HashMap<>();
            if (null != trsFiveYear && null != fieldDefineFiveYear) {
                //明细信息
                TrsFiveYearInf trsFiveYearInf = trsFiveYear.getTrsFiveYearInf();
                //获取是LIST.开始的定义的字段，这些字段是最近5年历史表现信息的集合
                List<FieldDefineEntity> subTrsFiveYearFieldDefineList = fieldDefineFiveYear.stream().filter((e) -> e.getPath().startsWith(CommonTypeConsts.START_LIST))
                        .collect(Collectors.toList());
                if (null != trsFiveYearInf) {
                    Map<String, List> trsFiveYearJsonMap = new HashMap<>();
                    //明细列表信息
                    List<TrsFiveYearDetail> trsFiveYearDetailList = trsFiveYearInf.getTrsFiveYearDetailList();
                    List<Map> trsFiveYearDetails = new ArrayList<>();
                    for (TrsFiveYearDetail trsFiveYearDetail : trsFiveYearDetailList) {
                        Map<String, String> trsFiveYearDetailMap = new HashMap<>();
                        for (FieldDefineEntity fieldDefineEntity : subTrsFiveYearFieldDefineList) {
                            String path = fieldDefineEntity.getPath();
                            if (path != null && path.startsWith(CommonTypeConsts.START_LIST)) {
                                path = path.replace(CommonTypeConsts.START_LIST, "");
                            }
                            String name = fieldDefineEntity.getName();
                            String type = fieldDefineEntity.getType();
                            String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(trsFiveYearDetail, path), type);
                            trsFiveYearDetailMap.put(name, value);
                        }
                        trsFiveYearDetails.add(trsFiveYearDetailMap);
                    }
                    if (null != trsFiveYearDetails && trsFiveYearDetails.size() > 0) {
                        trsFiveYearJsonMap.put("trsFiveYearList", trsFiveYearDetails);
                        trsFiveYearMap.putAll(trsFiveYearJsonMap);
                    }
                }
                List<FieldDefineEntity> fieldDefineFiveYearList = fieldDefineFiveYear.stream().filter((e) -> !e.getPath().startsWith(CommonTypeConsts.START_LIST))
                        .collect(Collectors.toList());
                for (FieldDefineEntity fieldDefineEntity : fieldDefineFiveYearList) {
                    String path = fieldDefineEntity.getPath();
                    String name = fieldDefineEntity.getName();
                    String type = fieldDefineEntity.getType();
                    String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(trsFiveYear, path), type);
                    trsFiveYearMap.put(name, value);
                }
                trsInfMap.put(CommonTypeConsts.TRSINFO_LASTFIVEYEAR, trsFiveYearMap);
            }
            //特殊交易信息
            TrsSpecInf trsSpecInf = trsAcctInf.getTrsSpecInf();
            Map<Object, Object> trsSpecInfMap = new HashMap<>();
            if (null != trsSpecInf && null != fieldDefineSpectrs) {
                for (FieldDefineEntity fieldDefineSpectr : fieldDefineSpectrs) {
                    String path = fieldDefineSpectr.getPath();
                    String name = fieldDefineSpectr.getName();
                    String type = fieldDefineSpectr.getType();
                    String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(trsSpecInf, path), type);
                    trsSpecInfMap.put(name, value);
                }
                trsInfMap.put(CommonTypeConsts.TRSINFO_SPECTRS, trsSpecInfMap);
            }
            //特殊事件说明信息
            TrsSpecEvtInf trsSpecEvtInf = trsAcctInf.getTrsSpecEvtInf();
            Map<String, String> trsSpecEvtInfMap = new HashMap<>();
            if (null != trsSpecEvtInf) {
                for (FieldDefineEntity fieldDefineEntity : fieldDefineSpecEvt) {
                    String path = fieldDefineEntity.getPath();
                    String name = fieldDefineEntity.getName();
                    String type = fieldDefineEntity.getType();
                    String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(trsSpecEvtInf, path), type);
                    trsSpecEvtInfMap.put(name, value);
                }
                trsInfMap.put(CommonTypeConsts.TRSINFO_SPECEVT, trsSpecEvtInfMap);
            }
            //大额专项分期信息段
            TrsSpecPrdInf trsSpecPrdInf = trsAcctInf.getTrsSpecPrdInf();
            Map<String, String> trsSpecPrdInfMap = new HashMap<>();
            if (null != trsSpecPrdInf && null != fieldDefineSpecPrd) {
                for (FieldDefineEntity fieldDefineEntity : fieldDefineSpecPrd) {
                    String path = fieldDefineEntity.getPath();
                    String name = fieldDefineEntity.getName();
                    String type = fieldDefineEntity.getType();
                    String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(trsSpecPrdInf, path), type);
                    trsSpecPrdInfMap.put(name, value);
                }
                trsInfMap.put(CommonTypeConsts.TRSINFO_SPECTRSPRD, trsSpecPrdInfMap);
            }
            trsInfoList.add(trsInfMap);
        }
        jsonResultMap.put(CommonTypeConsts.ACCT_LIST, trsInfoList);
        return jsonResultMap;
    }


    /**
     * 信息概要 查询记录概要
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> queryRecordsCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        SmryQueryRecord smryQueryRecord = jsonObject.toJavaObject(SmryQueryRecord.class);
        //上一次情况
        LastQueryRecord lastQueryRecord = smryQueryRecord.getLastQueryRecord();
        Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
        List<FieldDefineEntity> fieldDefineLasts = fieldDefineMap.get(CommonTypeConsts.QUERYRECORDS_LAST);
        for (FieldDefineEntity fieldDefineLast : fieldDefineLasts) {
            String path = fieldDefineLast.getPath();
            String name = fieldDefineLast.getName();
            String type = fieldDefineLast.getType();
            String value = "";
            value = ReflexUtils.getAnnotaionFiledValue(lastQueryRecord, path);
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        //最近信息情况
        LastMounthQueryRecord lastMounthQueryRecord = smryQueryRecord.getLastMounthQueryRecord();
        List<FieldDefineEntity> fieldDefineNears = fieldDefineMap.get(CommonTypeConsts.QUERYRECORDS_NEAR);
        for (FieldDefineEntity fieldDefineNear : fieldDefineNears) {
            String path = fieldDefineNear.getPath();
            String name = fieldDefineNear.getName();
            String type = fieldDefineNear.getType();
            String value = "";
            value = ReflexUtils.getAnnotaionFiledValue(lastMounthQueryRecord, path);
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        return jsonResultMap;
    }

    /**
     * 信息概要 公开信息汇总
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> publicInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        SmryPublic smryPublic = jsonObject.toJavaObject(SmryPublic.class);
        List<SmryPublicInf> smryPublicInfList = smryPublic.getSmryPublicInfList();

        Map<String, List<SmryPublicInf>> smryPublicInfMap = smryPublicInfList.stream().collect(Collectors.groupingBy(e -> e.getInfoTypeDesc()));

        Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
        //欠缴信息
        List<FieldDefineEntity> fieldDefineFees = fieldDefineMap.get(CommonTypeConsts.PUBLICINFO_FEE);

        List<SmryPublicInf> smryPublicInfFees = smryPublicInfMap.get(CommonTypeConsts.PUBLICINFO_FEE);
        for (FieldDefineEntity fieldDefineFee : fieldDefineFees) {
            setJsonResultMap(fieldDefineFee, jsonResultMap, smryPublicInfFees);
        }
        //判决信息
        List<FieldDefineEntity> fieldDefineJudgments = fieldDefineMap.get(CommonTypeConsts.PUBLICINFO_JUDGMENT);
        List<SmryPublicInf> smryPublicInfJudgments = smryPublicInfMap.get(CommonTypeConsts.PUBLICINFO_JUDGMENT);
        for (FieldDefineEntity fieldDefineJudgment : fieldDefineJudgments) {
            setJsonResultMap(fieldDefineJudgment, jsonResultMap, smryPublicInfJudgments);
        }

        //执行信息
        List<FieldDefineEntity> fieldDefineExecutes = fieldDefineMap.get(CommonTypeConsts.PUBLICINFO_EXECUTE);
        List<SmryPublicInf> smryPublicInfExecutes = smryPublicInfMap.get(CommonTypeConsts.PUBLICINFO_EXECUTE);
        for (FieldDefineEntity fieldDefineExecute : fieldDefineExecutes) {
            setJsonResultMap(fieldDefineExecute, jsonResultMap, smryPublicInfExecutes);
        }
        //惩戒类信息
        List<FieldDefineEntity> fieldDefinePunishments = fieldDefineMap.get(CommonTypeConsts.PUBLICINFO_PUNISHMENT);
        List<SmryPublicInf> smryPublicInfPunishments = smryPublicInfMap.get(CommonTypeConsts.PUBLICINFO_EXECUTE);
        for (FieldDefineEntity fieldDefinePunishment : fieldDefinePunishments) {
            setJsonResultMap(fieldDefinePunishment, jsonResultMap, smryPublicInfPunishments);
        }
        return jsonResultMap;
    }


    /**
     * 信息概要 非金融信用交易信息概要
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> noFinaInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();

        SmryNoFina smryNoFina = jsonObject.toJavaObject(SmryNoFina.class);
        List<SmryNoFinaInf> smryNoFinaInfList = smryNoFina.getSmryNoFinaInfList();
        Map<String, List<SmryNoFinaInf>> smryNoFinaInfMap = smryNoFinaInfList.stream().collect(Collectors.groupingBy(e -> e.getInfoTypeDesc()));
        List<SmryNoFinaInf> smryNoFinaInfs = smryNoFinaInfMap.get(CommonTypeConsts.NOFINAINFO_TYPE);

        Map<String, List<FieldDefineEntity>> defineEntityMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
        List<FieldDefineEntity> fieldDefineMap = defineEntityMap.get(CommonTypeConsts.NOFINAINFO_TYPE);
        for (FieldDefineEntity fieldDefineEntity : fieldDefineMap) {
            String path = fieldDefineEntity.getPath();
            String name = fieldDefineEntity.getName();
            String type = fieldDefineEntity.getType();
            String value = "";
            if (null != smryNoFinaInfs && smryNoFinaInfs.size() > 0) {
                value = ReflexUtils.getAnnotaionFiledValue(smryNoFinaInfs.get(0), path);
            }
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        return jsonResultMap;
    }

    /**
     * 信息概要 4. 公共信息概要
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> commInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        SmryComm smryComm = jsonObject.toJavaObject(SmryComm.class);
        List<SmryCommInfo> smryCommInfoList = smryComm.getSmryCommInfoList();

        Map<String, List<SmryCommInfo>> smryCommInfoMap = smryCommInfoList.stream().collect(Collectors.groupingBy(e -> e.getCommTypeDesc()));

        Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
        //欠税信息
        List<FieldDefineEntity> fieldDefineTaxs = fieldDefineMap.get(CommonTypeConsts.COMMINFO_TYPE_TAX);
        List<SmryCommInfo> smryCommInfoTaxs = smryCommInfoMap.get(CommonTypeConsts.COMMINFO_TYPE_TAX);
        for (FieldDefineEntity fieldDefineTax : fieldDefineTaxs) {
            String path = fieldDefineTax.getPath();
            String name = fieldDefineTax.getName();
            String type = fieldDefineTax.getType();
            String value = "";
            if (null != smryCommInfoTaxs && smryCommInfoTaxs.size() > 0) {
                value = ReflexUtils.getAnnotaionFiledValue(smryCommInfoTaxs.get(0), path);
            }
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }

        //民事判决信息
        List<FieldDefineEntity> fieldDefineCivil = fieldDefineMap.get(CommonTypeConsts.COMMINFO_TYPE_CIVIL);
        List<SmryCommInfo> smryCommInfoCivils = smryCommInfoMap.get(CommonTypeConsts.COMMINFO_TYPE_CIVIL);
        for (FieldDefineEntity fieldDefineEntity : fieldDefineCivil) {
            String path = fieldDefineEntity.getPath();
            String name = fieldDefineEntity.getName();
            String type = fieldDefineEntity.getType();
            String value = "";
            if (null != smryCommInfoCivils && smryCommInfoCivils.size() > 0) {
                value = ReflexUtils.getAnnotaionFiledValue(smryCommInfoCivils.get(0), path);
            }
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }

        //强制执行信息
        List<FieldDefineEntity> fieldDefineEnforces = fieldDefineMap.get(CommonTypeConsts.COMMINFO_TYPE_ENFORCE);
        List<SmryCommInfo> smryCommInfoEnforces = smryCommInfoMap.get(CommonTypeConsts.COMMINFO_TYPE_ENFORCE);
        for (FieldDefineEntity fieldDefineEnforce : fieldDefineEnforces) {
            String path = fieldDefineEnforce.getPath();
            String name = fieldDefineEnforce.getName();
            String type = fieldDefineEnforce.getType();
            String value = "";
            if (null != smryCommInfoEnforces && smryCommInfoEnforces.size() > 0) {
                value = ReflexUtils.getAnnotaionFiledValue(smryCommInfoEnforces.get(0), path);
            }
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        //行政处罚信息
        List<FieldDefineEntity> fieldDefineAdministratives = fieldDefineMap.get(CommonTypeConsts.COMMINFO_TYPE_ADMINISTRATIVE);
        List<SmryCommInfo> smryCommInfoAdministratives = smryCommInfoMap.get(CommonTypeConsts.COMMINFO_TYPE_ADMINISTRATIVE);

        for (FieldDefineEntity fieldDefineAdministrative : fieldDefineAdministratives) {
            String path = fieldDefineAdministrative.getPath();
            String name = fieldDefineAdministrative.getName();
            String type = fieldDefineAdministrative.getType();
            String value = "";
            if (null != smryCommInfoAdministratives && smryCommInfoAdministratives.size() > 0) {
                value = ReflexUtils.getAnnotaionFiledValue(smryCommInfoAdministratives.get(0), path);
            }
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }

        return jsonResultMap;
    }

    /**
     * 后付费业务欠费信息汇总
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> postPayInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        SmryNoCredit smryNoCredit = jsonObject.toJavaObject(SmryNoCredit.class);
        List<SmryNoCreditInf> smryNoCreditInfList = smryNoCredit.getSmryNoCreditInfList();
        //根据后付费业务描述进行分组
        Map<String, List<SmryNoCreditInf>> smryNoCreditInfoMap = smryNoCreditInfList.stream().collect(Collectors.groupingBy(e -> e.getPiTypeDesc()));

        List<SmryNoCreditInf> telecomSmryNoCreditInfs = smryNoCreditInfoMap.get("电信业务");

        List<SmryNoCreditInf> publicSmryNoCreditInfs = smryNoCreditInfoMap.get("水电气等公用事业");

        Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
        List<FieldDefineEntity> telecomFieldDefineList = fieldDefineMap.get("电信业务");
        for (FieldDefineEntity fieldDefineEntity : telecomFieldDefineList) {
            String path = fieldDefineEntity.getPath();
            String name = fieldDefineEntity.getName();
            String type = fieldDefineEntity.getType();
            String value = "";
            if (null != telecomSmryNoCreditInfs && telecomSmryNoCreditInfs.size() > 0) {
                value = ReflexUtils.getAnnotaionFiledValue(telecomSmryNoCreditInfs.get(0), path);
            }
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        List<FieldDefineEntity> publicFieldDefineList = fieldDefineMap.get("水电气等公用事业");
        for (FieldDefineEntity fieldDefineEntity : publicFieldDefineList) {
            String path = fieldDefineEntity.getPath();
            String name = fieldDefineEntity.getName();
            String type = fieldDefineEntity.getType();
            String value = "";
            if (null != publicSmryNoCreditInfs && publicSmryNoCreditInfs.size() > 0) {
                value = ReflexUtils.getAnnotaionFiledValue(publicSmryNoCreditInfs.get(0), path);
            }

            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        return jsonResultMap;
    }


    /**
     * 相关还款责任人计算
     *
     * @param jsonObject
     * @param defineEntityList
     * @return
     */
    public Map<String, String> relRepayInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        SmryRepay smryRepay = jsonObject.toJavaObject(SmryRepay.class);
        List<SmryRepayInfo> smryRepayInfoList = Optional.ofNullable(smryRepay.getSmryRepayInfoList()).orElse(new ArrayList<SmryRepayInfo>());
        //按照借款人身份类别进行分组
        Map<String, List<SmryRepayInfo>> repayInfoMap = smryRepayInfoList.
                stream()
                .filter((e) -> e.getBoTypeCode() != null)
                .collect(Collectors.groupingBy(e -> e.getBoTypeCode()));

        Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream()
                .filter((e) -> e.getParam() != null)
                .collect(Collectors.groupingBy(e -> e.getParam()));
        //个人
        List<FieldDefineEntity> perFieldDefineEntities = fieldDefineMap.get("1");
        //个人
        List<SmryRepayInfo> perSmryRepayInfos = repayInfoMap.get("1");
        List<SmryRepayInfo> perGuaranteeList = null;
        List<SmryRepayInfo> perOtherList = null;
        if (perSmryRepayInfos != null) {
            //按类别进行分组
            Map<String, List<SmryRepayInfo>> perRespCodeMap = perSmryRepayInfos.stream()
                    .filter((e) -> e.getRelPayRespDesc() != null)
                    .collect(Collectors.groupingBy(e -> e.getRelPayRespDesc()));
            perGuaranteeList = perRespCodeMap.get("担保责任");
            perOtherList = perRespCodeMap.get("其他相关还款责任");
        }
        for (FieldDefineEntity perFieldDefineEntity : perFieldDefineEntities) {
            String path = perFieldDefineEntity.getPath();
            String name = perFieldDefineEntity.getName();
            String type = perFieldDefineEntity.getType();
            String value = "";
            if (name.contains("Guarantee")) {
                if (perGuaranteeList != null && perGuaranteeList.size() > 0) {
                    value = ReflexUtils.getAnnotaionFiledValue(perGuaranteeList.get(0), path);
                }
            } else {
                if (perOtherList != null && perOtherList.size() > 0) {
                    value = ReflexUtils.getAnnotaionFiledValue(perOtherList.get(0), path);
                }
            }
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        //企业
        List<SmryRepayInfo> entSmryRepayInfos = repayInfoMap.get("2");
        List<SmryRepayInfo> entGuaranteeList = null;
        List<SmryRepayInfo> entOtherList = null;
        if (entSmryRepayInfos != null) {
            Map<String, List<SmryRepayInfo>> entRespCodeMap = entSmryRepayInfos.stream()
                    .filter((e) -> e.getRelPayRespDesc() != null)
                    .collect(Collectors.groupingBy(e -> e.getRelPayRespDesc()));
            entGuaranteeList = entRespCodeMap.get("担保责任");
            entOtherList = entRespCodeMap.get("其他相关还款责任");
        }
        List<FieldDefineEntity> entFieldDefineEntities = fieldDefineMap.get("2");
        for (FieldDefineEntity entFieldDefineEntity : entFieldDefineEntities) {
            String path = entFieldDefineEntity.getPath();
            String name = entFieldDefineEntity.getName();
            String type = entFieldDefineEntity.getType();
            String value = "";
            if (name.contains("Guarantee")) {
                if (null != entGuaranteeList && entGuaranteeList.size() > 0) {
                    value = ReflexUtils.getAnnotaionFiledValue(entGuaranteeList.get(0), path);
                }
            } else {
                if (null != entOtherList && entOtherList.size() > 0) {
                    value = ReflexUtils.getAnnotaionFiledValue(entOtherList.get(0), path);
                }
            }
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        return jsonResultMap;
    }

    /**
     * 信息概要. 2.2 被追偿信息汇总
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> recoveredInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        SmryRecovered smryRecovered = jsonObject.toJavaObject(SmryRecovered.class);
        //信贷交易信息提示配置参数
        Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
        List<SmryRecoveredInf> smryRecoveredInfList = new ArrayList<>();
        if (null != smryRecovered) {
            SmryRecoveredList smryRecoveredList = smryRecovered.getSmryRecoveredList();
            if (null != smryRecoveredList) {
                smryRecoveredInfList = smryRecoveredList.getSmryRecoveredInfList();
            }
        }
        Map<String, List<SmryRecoveredInf>> smryRecoveredInfMap = smryRecoveredInfList.stream().collect(Collectors.groupingBy(e -> e.getBusTypeDesc()));
        //计算合计
        List<FieldDefineEntity> fieldDefineEntitiesSum = fieldDefineMap.get("@sum");
        for (FieldDefineEntity fieldDefineEntity : fieldDefineEntitiesSum) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            String type = fieldDefineEntity.getType();
            String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(smryRecovered, path), type);
            jsonResultMap.put(name, value);
        }

        //资产处置业务
        List<FieldDefineEntity> fieldDefineEntitiesAsset = fieldDefineMap.get("资产处置业务");
        List<SmryRecoveredInf> smryRecoveredInfAssets = smryRecoveredInfMap.get("资产处置业务");
        for (FieldDefineEntity fieldDefineEntity : fieldDefineEntitiesAsset) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            String type = fieldDefineEntity.getType();
            String value = "";
            if (null != smryRecoveredInfAssets && smryRecoveredInfAssets.size() > 0) {
                value = ReflexUtils.getAnnotaionFiledValue(smryRecoveredInfAssets.get(0), path);
            }
            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        //资产处置业务
        List<FieldDefineEntity> fieldDefineEntitiesAdvances = fieldDefineMap.get("垫款业务");
        List<SmryRecoveredInf> smryRecoveredInfAdvances = smryRecoveredInfMap.get("垫款业务");
        for (FieldDefineEntity fieldDefineEntity : fieldDefineEntitiesAdvances) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            String type = fieldDefineEntity.getType();
            String value = "";
            if (null != smryRecoveredInfAdvances && smryRecoveredInfAdvances.size() > 0) {
                value = ReflexUtils.getAnnotaionFiledValue(smryRecoveredInfAdvances.get(0), path);
            }

            jsonResultMap.put(name, BaseUtils.getValueByType(value, type));
        }
        return jsonResultMap;
    }

    /**
     * 信息概要. 2.1 信贷交易信息提示
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> creditTradeTipsCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        SmryTradeTips smryTradeTips = jsonObject.toJavaObject(SmryTradeTips.class);
        SmryTradeTipsList smryTradeTipsList = smryTradeTips.getSmryTradeTipsList();
        //信贷交易信息提示配置参数
        Map<String, List<FieldDefineEntity>> fieldDefineMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));

        List<SmryTradeTipsInfo> smryTradeTipsInfo = smryTradeTipsList.getSmryTradeTipsInfo();
        Map<String, List<SmryTradeTipsInfo>> smryTradeTipsInfoMap = smryTradeTipsInfo.stream()
                .collect(Collectors.groupingBy((e) -> e.getBusTypeDesc()));
        //key = 11 个人住房贷款
        List<FieldDefineEntity> fieldDefines11 = fieldDefineMap.get("个人住房贷款");
        List<SmryTradeTipsInfo> smryTradeTipsInfos11 = smryTradeTipsInfoMap.get("个人住房贷款");

        for (FieldDefineEntity fieldDefineEntity : fieldDefines11) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            setCreditTradeTipsValue(path, name, smryTradeTipsInfos11, jsonResultMap);
        }

        //key = 12 个人商用贷款
        List<FieldDefineEntity> fieldDefines12 = fieldDefineMap.get("个人商用房贷款（包括商住两用房）");
        List<SmryTradeTipsInfo> smryTradeTipsInfos12 = smryTradeTipsInfoMap.get("个人住房贷款");
        for (FieldDefineEntity fieldDefineEntity : fieldDefines12) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            setCreditTradeTipsValue(path, name, smryTradeTipsInfos12, jsonResultMap);
        }
        //key = 19 其它类贷款
        List<FieldDefineEntity> fieldDefines19 = fieldDefineMap.get("其他类贷款");
        List<SmryTradeTipsInfo> smryTradeTipsInfos19 = smryTradeTipsInfoMap.get("其他类贷款");
        for (FieldDefineEntity fieldDefineEntity : fieldDefines19) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            setCreditTradeTipsValue(path, name, smryTradeTipsInfos19, jsonResultMap);
        }
        //key = 21 贷记卡
        List<FieldDefineEntity> fieldDefines21 = fieldDefineMap.get("贷记卡");
        List<SmryTradeTipsInfo> smryTradeTipsInfos21 = smryTradeTipsInfoMap.get("贷记卡");
        for (FieldDefineEntity fieldDefineEntity : fieldDefines21) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            setCreditTradeTipsValue(path, name, smryTradeTipsInfos21, jsonResultMap);
        }
        //key = 22 准贷记卡
        List<FieldDefineEntity> fieldDefines22 = fieldDefineMap.get("准贷记卡");
        List<SmryTradeTipsInfo> smryTradeTipsInfos22 = smryTradeTipsInfoMap.get("准贷记卡");
        for (FieldDefineEntity fieldDefineEntity : fieldDefines22) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            setCreditTradeTipsValue(path, name, smryTradeTipsInfos22, jsonResultMap);
        }
        //key = 9 其它
        List<FieldDefineEntity> fieldDefines9 = fieldDefineMap.get("未知");
        List<SmryTradeTipsInfo> smryTradeTipsInfos9 = smryTradeTipsInfoMap.get("未知");
        for (FieldDefineEntity fieldDefineEntity : fieldDefines9) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            setCreditTradeTipsValue(path, name, smryTradeTipsInfos9, jsonResultMap);
        }

        return jsonResultMap;
    }

    /**
     * 设置各个值
     *
     * @param path               数据源的路径
     * @param name               数据对应的key
     * @param smryTradeTipsInfos 数据来源结果
     * @param jsonResultMap      计算结果Map
     */
    private void setCreditTradeTipsValue(String path, String name, List<SmryTradeTipsInfo> smryTradeTipsInfos, Map jsonResultMap) {
        String accNum = "";
        String firstBusMonth = "";
        if (null != smryTradeTipsInfos && smryTradeTipsInfos.size() > 0) {
            accNum = Optional.ofNullable(smryTradeTipsInfos.get(0).getAccNum()).orElse("0");
            firstBusMonth = smryTradeTipsInfos.get(0).getFirstBusMonth();
        }
        if (path.equals("ACCOUNTNUM")) {
            jsonResultMap.put(name, accNum);
        }
        if (path.equals("FIRSTBUSMONTH")) {
            jsonResultMap.put(name, firstBusMonth);
        }
    }

    /**
     * 个人信用报告“数字解读”
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> gradeInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        SmryNumRead smryNumRead = jsonObject.toJavaObject(SmryNumRead.class);
        //把说明信息段 合并成一条
        List<SmryNumReadInf> smryNumReadInfList = smryNumRead.getSmryNumReadInfList();
        String scoreExplain = smryNumReadInfList.stream().map(SmryNumReadInf::getScoreExplain)
                .collect(Collectors.joining(";"));

        defineEntityList.stream().forEach((e) -> {
            String name = e.getName();
            String path = e.getPath();
            if (name.equals("scoreExplain")) {
                jsonResultMap.put(name, scoreExplain);
            } else {
                String value = ReflexUtils.getAnnotaionFiledValue(smryNumRead, path);
                jsonResultMap.put(name, value);
            }
        });
        return jsonResultMap;
    }


    /**
     * 职业信息
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> vocationInfoCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        BaseVocation baseVocation = jsonObject.toJavaObject(BaseVocation.class);
        List<BaseVocationInf> baseVocationInfList = baseVocation.getBaseVocationInfList();
        //按更新日期获取最新的职业信息
        BaseVocationInf baseVocationInf = baseVocationInfList.stream().sorted(
                new Comparator<BaseVocationInf>() {
                    @Override
                    public int compare(BaseVocationInf o1, BaseVocationInf o2) {
                        Date date1 = DateUtils.convertStrToDate(o1.getUpdateDateInfo(), DateUtils.YYYY_MM_DD);
                        Date date2 = DateUtils.convertStrToDate(o2.getUpdateDateInfo(), DateUtils.YYYY_MM_DD);
                        return date2.compareTo(date1);
                    }
                }

        ).findFirst().get();
        defineEntityList.stream().forEach((e) -> {
            String name = e.getName();
            String path = e.getPath();
            String value = ReflexUtils.getAnnotaionFiledValue(baseVocationInf, path);
            jsonResultMap.put(name, value);
        });

        return jsonResultMap;
    }

    /**
     * 居住信息
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, String> residenceCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, String> jsonResultMap = new HashMap<>();
        BaseResidence baseResidence = jsonObject.toJavaObject(BaseResidence.class);
        List<BaseResidenceInf> baseResidenceInfs = baseResidence.getBaseResidenceInfs();
        //按更新日期获取最新的居住信息
        BaseResidenceInf baseResidenceInf = baseResidenceInfs.stream().sorted(
                new Comparator<BaseResidenceInf>() {
                    @Override
                    public int compare(BaseResidenceInf o1, BaseResidenceInf o2) {
                        Date date1 = DateUtils.convertStrToDate(o1.getUpdateDateInfo(), DateUtils.YYYY_MM_DD);
                        Date date2 = DateUtils.convertStrToDate(o2.getUpdateDateInfo(), DateUtils.YYYY_MM_DD);
                        return date2.compareTo(date1);
                    }
                }

        ).findFirst().get();
        defineEntityList.stream().forEach((e) -> {
            String name = e.getName();
            String path = e.getPath();
            String value = ReflexUtils.getAnnotaionFiledValue(baseResidenceInf, path);
            jsonResultMap.put(name, value);
        });

        return jsonResultMap;
    }

    /**
     * 获取手机号码
     *
     * @param jsonObject       xml解析过来的手机号码
     * @param defineEntityList 数据库里配置的手机号码信息
     * @return
     */
    public Map<String, Object> phoneCalculate(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, Object> jsonResultMap = new HashMap<>();
        BasePhone basePhone = jsonObject.toJavaObject(BasePhone.class);
        List<BasePhoneInf> basePhoneInfList = basePhone.getBasePhoneInfList();
        List<Object> phoneList = new ArrayList<>();
        basePhoneInfList.stream().forEach((e) -> {
            HashMap<String, String> phoneMap = new HashMap<>();
            for (FieldDefineEntity fieldDefineEntity : defineEntityList) {
                String path = fieldDefineEntity.getPath();
                String name = fieldDefineEntity.getName();
                String type = fieldDefineEntity.getType();
                String value = BaseUtils.getValueByType(ReflexUtils.getAnnotaionFiledValue(e, path), type);
                phoneMap.put(name, value);
            }
            phoneList.add(phoneMap);
        });
        jsonResultMap.put(CommonTypeConsts.PHONE_LIST, phoneList);
        return jsonResultMap;
    }

    /**
     * 信息概要. 2.4 逾期（透支）汇总信息段
     *
     * @param jsonObject
     * @return
     */
    public Map<String, String> nonCycAccSum(JSONObject jsonObject, List<FieldDefineEntity> defineEntityList) {
        Map<String, List<FieldDefineEntity>> fieldDefineEntityMap = defineEntityList.stream().collect(Collectors.groupingBy(e -> e.getParam()));
        Overdraft overdraft = jsonObject.toJavaObject(Overdraft.class);
        List<OverdraftInfo> overdraftInfoList = overdraft.getOverdraftInfoList();
        //解析出来的数据进行分组
        Map<String, List<OverdraftInfo>> overDraftInfoMap = overdraftInfoList.stream().collect(Collectors.groupingBy((e) -> e.getAccTypeCode()));

        //从数据库里获取到的配置的数据====
        //key=1非循环贷
        List<FieldDefineEntity> fieldDefineEntities1 = fieldDefineEntityMap.get("1");
        List<OverdraftInfo> overdraftInfos1 = overDraftInfoMap.get("1");
        Map<String, String> jsonResultMap = new HashMap<>();
        for (FieldDefineEntity fieldDefineEntity : fieldDefineEntities1) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            getValue(path, name, jsonResultMap, overdraftInfos1);
        }
        //key=2 循环额度下分账户
        List<FieldDefineEntity> fieldDefineEntities2 = fieldDefineEntityMap.get("2");
        List<OverdraftInfo> overdraftInfos2 = overDraftInfoMap.get("2");
        for (FieldDefineEntity fieldDefineEntity : fieldDefineEntities2) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            getValue(path, name, jsonResultMap, overdraftInfos2);
        }
        //key=3 循环贷账户
        List<FieldDefineEntity> fieldDefineEntities3 = fieldDefineEntityMap.get("3");
        List<OverdraftInfo> overdraftInfos3 = overDraftInfoMap.get("3");
        for (FieldDefineEntity fieldDefineEntity : fieldDefineEntities3) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            getValue(path, name, jsonResultMap, overdraftInfos3);
        }
        //key=4 贷记卡账户
        List<FieldDefineEntity> fieldDefineEntities4 = fieldDefineEntityMap.get("4");
        List<OverdraftInfo> overdraftInfos4 = overDraftInfoMap.get("4");
        for (FieldDefineEntity fieldDefineEntity : fieldDefineEntities4) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            getValue(path, name, jsonResultMap, overdraftInfos4);
        }
        //key=5 准贷记卡账户
        List<FieldDefineEntity> fieldDefineEntities5 = fieldDefineEntityMap.get("5");
        List<OverdraftInfo> overdraftInfos5 = overDraftInfoMap.get("5");
        for (FieldDefineEntity fieldDefineEntity : fieldDefineEntities5) {
            String name = fieldDefineEntity.getName();
            String path = fieldDefineEntity.getPath();
            getValue(path, name, jsonResultMap, overdraftInfos5);
        }
        return jsonResultMap;
    }

    /**
     * 进行赋值
     *
     * @param path
     * @param name
     * @param jsonResultMap
     * @param overdraftInfos
     */
    private void getValue(String path, String name, Map<String, String> jsonResultMap, List<OverdraftInfo> overdraftInfos) {

        if (path.equals("ACCSUM")) {
            jsonResultMap.put(name, String.valueOf(Optional.ofNullable(overdraftInfos.get(0).getAccSum()).orElse(0)));
        }
        if (path.equals("MONTHSUM")) {
            jsonResultMap.put(name, String.valueOf(Optional.ofNullable(overdraftInfos.get(0).getMonthSum()).orElse(0)));
        }
        if (path.equals("OVERDRAFTMAX")) {
            jsonResultMap.put(name, String.valueOf(Optional.ofNullable(overdraftInfos.get(0).getOverdraftMax()).orElse(new BigDecimal("0"))));
        }
        if (path.equals("OVERMONTH")) {
            jsonResultMap.put(name, String.valueOf(Optional.ofNullable(overdraftInfos.get(0).getOverMonth()).orElse(0)));
        }
    }

}
