package com.kexun;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.kexun.model.xml.*;
import com.kexun.utils.DateUtils;
import com.kexun.utils.ParseXmlUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.junit.jupiter.api.Test;
import org.springframework.util.StringUtils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.zip.GZIPInputStream;

/**
 * Description:
 *
 * @author lgs
 * @date 2023/05/30
 */
@Slf4j
public class ParseXmlTest {

    private String credit_trade_tips_smry;

    @Test
    public void test() throws Exception {
        String xmlStr = "<CREDITREPORT><OTHERIDSGLS><OTHERIDSG><CERTTYPECODE>证件类型代码</CERTTYPECODE><CERTTYPEDESC>证件类型描述</CERTTYPEDESC><CERTNUMBER>证件号码</CERTNUMBER></OTHERIDSG><OTHERIDSG><CERTTYPECODE>证件类型代码1</CERTTYPECODE><CERTTYPEDESC>证件类型描述1</CERTTYPEDESC><CERTNUMBER>证件号码1</CERTNUMBER></OTHERIDSG></OTHERIDSGLS></CREDITREPORT>";
        //创建SAXReader的对象
        SAXReader saxReader = new SAXReader();
        Element rootElement = null;
        //通过reader对象的read方法加载xml文件，获取document对象
        Document document = saxReader.read(new ByteArrayInputStream(xmlStr.getBytes(StandardCharsets.UTF_8)));
        //通过document对象获取根节点
        rootElement = document.getRootElement();
        String nodeName = "OTHERIDSGLS";
        //得到这个信息节点的节点信息
        Element nodeNameElement = rootElement.element(nodeName);
        List<Element> elements = nodeNameElement.elements();
        for (Element element : elements) {
            List<Element> subElement = element.elements();

            for (Element subNode : subElement) {
                String name = subNode.getName();
                String text = subNode.getText();
                log.info(name + ":" + text);
            }
        }
        System.out.println(elements.size());
    }

    /**
     * 数组解读
     *
     * @throws Exception
     */
    @Test
    public void testFenShu() throws Exception {
        String fenShu = "<CREDITREPORT><GRADEINFOSMRY><NUMBERREAD>数字解读 个人信用评分分值（取值范围 0-1000）</NUMBERREAD><RELPOSITION>相对位置</RELPOSITION><SCOREEXPLAINNUM>分数说明条数</SCOREEXPLAINNUM><SCOREEXPLAINCODELS><SCOREEXPLAINCODE>影响因素1 ：存在逾期还款记录</SCOREEXPLAINCODE><SCOREEXPLAINCODE>影响因素2 ：无</SCOREEXPLAINCODE></SCOREEXPLAINCODELS></GRADEINFOSMRY></CREDITREPORT>";
        //创建SAXReader的对象
        SAXReader saxReader = new SAXReader();
        Element rootElement = null;
        //通过reader对象的read方法加载xml文件，获取document对象
        Document document = saxReader.read(new ByteArrayInputStream(fenShu.getBytes(StandardCharsets.UTF_8)));
        //通过document对象获取根节点
        rootElement = document.getRootElement();
        String nodeName = "GRADEINFOSMRY";
        //得到这个信息节点的节点信息
        Element nodeNameElement = rootElement.element(nodeName);
        //遍历所有的子节点
        for (Iterator iter = nodeNameElement.elementIterator(); iter.hasNext(); ) {
            Element element = (Element) iter.next();

            if (element.elements().size() > 0) {
                //继续循环
                List<Element> elements = element.elements();
                for (Element sub : elements) {
                    log.info(sub.getName() + ":" + sub.getText());
                }
            } else {
                log.info(element.getName() + ":" + element.getText());
            }
        }
    }

    @Test
    public void textJson() throws Exception{

        String s = System.getProperty("user.dir") + File.separator + "src"+File.separator+"test"+File.separator+"resources" + File.separator;
        String fileName = "json.txt";
        File file = new File(s + fileName);
        String json = FileUtils.readFileToString(file, StandardCharsets.UTF_8);

        String fenShu = "<CREDITREPORT><GRADEINFOSMRY><NUMBERREAD>数字解读 个人信用评分分值（取值范围 0-1000）</NUMBERREAD><RELPOSITION>相对位置</RELPOSITION><SCOREEXPLAINNUM>分数说明条数</SCOREEXPLAINNUM><SCOREEXPLAINCODELS><SCOREEXPLAINCODE>影响因素1 ：存在逾期还款记录</SCOREEXPLAINCODE><SCOREEXPLAINCODE>影响因素2 ：无</SCOREEXPLAINCODE></SCOREEXPLAINCODELS></GRADEINFOSMRY></CREDITREPORT>";

        JSONObject jsonObject = (JSONObject)JSONObject.parse(json);

        jsonObject.size();
        for (Map.Entry<String, Object> entry: jsonObject.entrySet()) {
            Object object = entry.getValue();

        }

    }

    /**
     *  xml转json
     * @throws Exception
     */
    @Test
    public void textXmlToJson() throws Exception{

        String path = System.getProperty("user.dir") + File.separator + "src"+File.separator+"test"+File.separator+"resources" + File.separator;
        String fileName = "credit.xml";
        File file = new File(path + fileName);
        String xmlContext = FileUtils.readFileToString(file, StandardCharsets.UTF_8);

        JSONObject jsonObject = ParseXmlUtils.xmlToJson(xmlContext);

        log.info(jsonObject.toJSONString());

        for (Map.Entry<String, Object> entry: jsonObject.entrySet()){
            log.info(entry.getKey()+";");
            //把entry转化成JSONObject
            JSONObject o = (JSONObject)JSONObject.toJSON(entry);
            SmryTradeTips smryTradeTips = o.toJavaObject(SmryTradeTips.class);
            JSONObject object =(JSONObject)entry.getValue();
            String s = ((JSONObject) entry.getValue()).toJSONString();
            System.out.println(s);
            SmryTradeTips smryTradeTips1 = object.toJavaObject(SmryTradeTips.class);
            System.out.println(o.toJSONString());
        }
        log.info(jsonObject.size()+"");
    }

    @Test
    public void testGzip() throws Exception{
        String path = System.getProperty("user.dir") + File.separator + "src"+File.separator+"test"+File.separator+"resources" + File.separator;
        String fileName = "body.txt";
        File file = new File(path + fileName);
        String body = FileUtils.readFileToString(file, StandardCharsets.UTF_8);

        if (!StringUtils.isEmpty(body)) {
            byte[] decode = Base64.getDecoder().decode(body);
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            ByteArrayInputStream in = new ByteArrayInputStream(decode);
            try {
                GZIPInputStream ungzip = new GZIPInputStream(in);
                byte[] buffer = new byte[256];
                int n;
                while((n = ungzip.read(buffer)) >= 0) {
                    out.write(buffer, 0, n);
                }
            } catch (IOException e) {
                log.error("解压出现错误");
            }
            String xmlContext = out.toString("utf-8");
            log.info(xmlContext);
        }
    }


    @Test
    public void textXmlToJson1() throws Exception{

        String path = System.getProperty("user.dir") + File.separator + "src"+File.separator+"test"+File.separator+"resources" + File.separator;
        String fileName = "credit1.xml";
        File file = new File(path + fileName);
        String xmlContext = FileUtils.readFileToString(file, StandardCharsets.UTF_8);

        JSONObject jsonObject = ParseXmlUtils.xmlToJson(xmlContext);

        log.info(jsonObject.toJSONString());

        for (Map.Entry<String, Object> entry: jsonObject.entrySet()){
            log.info(entry.getKey()+";");
            //把entry转化成JSONObject
            JSONObject o = (JSONObject)JSONObject.toJSON(entry);
            String s1 = o.toJSONString();
            String s = ((JSONObject) entry.getValue()).toJSONString();
            System.out.println(s);
            SmryTradeTips smryTradeTips = JSONObject.parseObject(s, SmryTradeTips.class);
            System.out.println(o.toJSONString());
        }
        log.info(jsonObject.size()+"");
    }

    @Test
    public void test3(){
        String jsonTest = "{\"Name\":\"张三\",\"CTINFOLIST\":{\"CREDITTIPSINFO\":[{\"BUSITYPEDESC\":\"个人住房贷款\",\"BUDLDESC\":\"贷款\",\"BUSITYPECODE\":\"11\",\"BUDLCODE\":\"1\"},{\"BUSITYPEDESC\":\"个人商用房贷款（包括商住两用房）\",\"BUDLDESC\":\"贷款\",\"BUSITYPECODE\":\"12\",\"BUDLCODE\":\"1\"},{\"ACCOUNTNUM\":\"30\",\"FIRSTBUSMONTH\":\"2019-04\",\"BUSITYPEDESC\":\"其他类贷款\",\"BUDLDESC\":\"贷款\",\"BUSITYPECODE\":\"19\",\"BUDLCODE\":\"1\"},{\"ACCOUNTNUM\":\"11\",\"FIRSTBUSMONTH\":\"2014-03\",\"BUSITYPEDESC\":\"贷记卡\",\"BUDLDESC\":\"信用卡\",\"BUSITYPECODE\":\"21\",\"BUDLCODE\":\"2\"},{\"BUSITYPEDESC\":\"准贷记卡\",\"BUDLDESC\":\"信用卡\",\"BUSITYPECODE\":\"22\",\"BUDLCODE\":\"2\"},{\"BUSITYPEDESC\":\"未知\",\"BUDLDESC\":\"其它\",\"BUDLCODE\":\"9\"}]}}";

        TestXMl testXMl = JSONObject.parseObject(jsonTest, TestXMl.class);
        System.out.println(testXMl);
    }

    @Test
    public void test4(){

        String json = "{\"Name\":\"张三\",\"HEADBO\":{\"CRETYPECODE\":\"10\",\"DISSNUM\":\"0\",\"QUERYREASONDESC\":\"贷款审批\",\"QUERYREASONCODE\":\"02\",\"RPTTIME\":\"2023-01-10T11:03:31\",\"CRENUMBER\":\"320923199112061238\",\"RPTNO\":\"2023011011033153687083\",\"NAME\":\"刘飞\",\"QUERYORGCODE\":\"天津华北创业小额贷款有限公司\"},\"CTINFOLIST\":{\"CREDITTIPSINFO\":[{\"BUSITYPEDESC\":\"个人住房贷款\",\"BUDLDESC\":\"贷款\",\"BUSITYPECODE\":\"11\",\"BUDLCODE\":\"1\"},{\"BUSITYPEDESC\":\"个人商用房贷款（包括商住两用房）\",\"BUDLDESC\":\"贷款\",\"BUSITYPECODE\":\"12\",\"BUDLCODE\":\"1\"},{\"ACCOUNTNUM\":\"30\",\"FIRSTBUSMONTH\":\"2019-04\",\"BUSITYPEDESC\":\"其他类贷款\",\"BUDLDESC\":\"贷款\",\"BUSITYPECODE\":\"19\",\"BUDLCODE\":\"1\"},{\"ACCOUNTNUM\":\"11\",\"FIRSTBUSMONTH\":\"2014-03\",\"BUSITYPEDESC\":\"贷记卡\",\"BUDLDESC\":\"信用卡\",\"BUSITYPECODE\":\"21\",\"BUDLCODE\":\"2\"},{\"BUSITYPEDESC\":\"准贷记卡\",\"BUDLDESC\":\"信用卡\",\"BUSITYPECODE\":\"22\",\"BUDLCODE\":\"2\"},{\"BUSITYPEDESC\":\"未知\",\"BUDLDESC\":\"其它\",\"BUDLCODE\":\"9\"}]}}";
        JSONObject jsonObject = JSONObject.parseObject(json);
        JSONObject headbo = (JSONObject)jsonObject.get("HEADBO");
        String name = String.valueOf(headbo.get("NAME"));
        String creTypeCode = String.valueOf(headbo.get("CRETYPECODE"));
        String creTypeDesc = String.valueOf(headbo.get("CRETYPEDESC"));
        String creNumber = String.valueOf(headbo.get("CRENUMBER"));

        System.out.println(name);
        System.out.println(creTypeCode);
        System.out.println(creTypeDesc);
        System.out.println(creNumber);

    }

    @Test
    public void test7() throws Exception{
        HeadInfo headInfo = new HeadInfo();
        headInfo.setRptTime("2023-01-10T11:03:31");

        Class<? extends HeadInfo> aClass = headInfo.getClass();
        Field field = aClass.getDeclaredField("rptTime");
        field.setAccessible(true);
        Class<?> type = field.getType();
        String name = field.getName();
        String datetime = field.get(headInfo).toString();
        String s = DateUtils.convertTDate(datetime);

        String types = field.getGenericType().toString();
        System.out.println(types);

    }

    /**
     * 测试报告头. 其他身份标识信息段
     */
    @Test
    public void test8(){
        String json = "{\"OTHERIDSG\":[{\"CERTTYPECODE\":\"证件类型代码1\",\"CERTTYPEDESC\":\"证件类型描述1\",\"CERTNUMBER\":\"证件号码1\"},{\"CERTTYPECODE\":\"证件类型代码2\",\"CERTTYPEDESC\":\"证件类型描述2\",\"CERTNUMBER\":\"证件号码2\"}]}";
        HeadOther headOther = JSONObject.parseObject(json, HeadOther.class);
        System.out.println(headOther);

    }

    @Test
    public void test9() throws Exception{
        Overdraft overdraft = new Overdraft();
        List<OverdraftInfo> list = new ArrayList<>();
        OverdraftInfo overdraftInfo = new OverdraftInfo();
        overdraftInfo.setOverdraftMax(new BigDecimal("1111"));
        overdraftInfo.setAccSum(4);
        overdraftInfo.setAccTypeCode("122");
        list.add(overdraftInfo);

        OverdraftInfo overdraftInfo1 = new OverdraftInfo();
        overdraftInfo1.setOverdraftMax(new BigDecimal("2222"));
        overdraftInfo1.setAccSum(5);
        overdraftInfo1.setAccTypeCode("3333");
        list.add(overdraftInfo1);

        overdraft.setOverdraftInfoList(list);
        Class<? extends Overdraft> aClass = overdraft.getClass();

        Field[] declaredFields = aClass.getDeclaredFields();
        for (Field field : declaredFields) {
            field.setAccessible(true);
            Class<?> type = field.getType();
            if (type.equals(List.class)){
                String name = field.getDeclaredAnnotation(JSONField.class).name();
                if ("OVERDRAFTSUMSG".equals(name)){
                    List list1 = (List)field.get(overdraft);
                    System.out.println(list1);
                    for (Object object : list1) {
                        Class<?> aClass1 = object.getClass();

                    }
                }
            }
        }


    }
}
